@@ -32,7 +32,7 @@ public:
     ~ConfigHandler() override = default;
 
     void setConfig(KScreen::ConfigPtr config);
-    void updateInitialConfig();
+    void updateInitialData();
 
     OutputModel* outputModel() const {
         return m_outputs;
@@ -47,6 +47,17 @@ public:
     int retention() const;
     void setRetention(int retention);
 
+    qreal scale(const KScreen::OutputPtr &output) const;
+    void setScale(KScreen::OutputPtr &output, qreal scale);
+
+    KScreen::OutputPtr replicationSource(const KScreen::OutputPtr &output) const;
+    void setReplicationSource(KScreen::OutputPtr &output, const KScreen::OutputPtr &source);
+
+    bool autoRotate(const KScreen::OutputPtr &output) const;
+    void setAutoRotate(KScreen::OutputPtr &output, bool autoRotate);
+    bool autoRotateOnlyInTabletMode(const KScreen::OutputPtr &output) const;
+    void setAutoRotateOnlyInTabletMode(KScreen::OutputPtr &output, bool value);
+
     void writeControl();
 
     void checkNeedsSave();
@@ -66,12 +77,14 @@ private:
     void primaryOutputSelected(int index);
     void primaryOutputChanged(const KScreen::OutputPtr &output);
     void initOutput(const KScreen::OutputPtr &output);
+    void resetScale(const KScreen::OutputPtr &output);
 
     KScreen::ConfigPtr m_config = nullptr;
     KScreen::ConfigPtr m_initialConfig;
     OutputModel *m_outputs = nullptr;
 
     std::unique_ptr<ControlConfig> m_control;
+    std::unique_ptr<ControlConfig> m_initialControl;
     Control::OutputRetention m_initialRetention = Control::OutputRetention::
                                                   Undefined;
     QSize m_lastNormalizedScreenSize;
