@@ -49,6 +49,8 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return Utils::outputName(output);
     case EnabledRole:
         return output->isEnabled();
+    case InternalRole:
+        return output->type() == KScreen::Output::Type::Panel;
     case PrimaryRole:
         return output->isPrimary();
     case SizeRole:
@@ -57,6 +59,10 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return m_outputs[index.row()].pos;
     case NormalizedPositionRole:
         return output->geometry().topLeft();
+    case AutoRotateRole:
+        return m_config->autoRotate(output);
+    case AutoRotateOnlyInTabletModeRole:
+        return m_config->autoRotateOnlyInTabletMode(output);
     case RotationRole:
         return output->rotation();
     case ScaleRole:
@@ -70,7 +76,7 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
     case ReplicationSourceModelRole:
         return replicationSourceModel(output);
     case ReplicationSourceIndexRole:
-        return replicationSourceIndex(index.row(), output->replicationSource());
+        return replicationSourceIndex(index.row());
     case ReplicasModelRole:
         return replicasModel(output);
     case RefreshRatesRole:
@@ -90,7 +96,7 @@ bool OutputModel::setData(const QModelIndex &index,
         return false;
     }
 
-    const Output &output = m_outputs[index.row()];
+    Output &output = m_outputs[index.row()];
     switch (role) {
     case PositionRole:
         if (value.canConvert<QPoint>()) {
@@ -133,6 +139,16 @@ bool OutputModel::setData(const QModelIndex &index,
             return setRefreshRate(index.row(), value.toInt());
         }
         break;
+    case AutoRotateRole:
+        if (value.canConvert<bool>()) {
+            return setAutoRotate(index.row(), value.value<bool>());
+        }
+        break;
+    case AutoRotateOnlyInTabletModeRole:
+        if (value.canConvert<bool>()) {
+            return setAutoRotateOnlyInTabletMode(index.row(), value.value<bool>());
+        }
+        break;
     case RotationRole:
         if (value.canConvert<KScreen::Output::Rotation>()) {
             return setRotation(index.row(),
@@ -149,6 +165,7 @@ bool OutputModel::setData(const QModelIndex &index,
         const qreal scale = value.toReal(&ok);
         if (ok && !qFuzzyCompare(output.ptr->scale(), scale)) {
             output.ptr->setScale(scale);
+            m_config->setScale(output.ptr, scale);
             Q_EMIT sizeChanged();
             Q_EMIT dataChanged(index, index, {role, SizeRole});
             return true;
@@ -161,10 +178,13 @@ bool OutputModel::setData(const QModelIndex &index,
 QHash<int, QByteArray> OutputModel::roleNames() const {
     QHash<int, QByteArray> roles = QAbstractItemModel::roleNames();
     roles[EnabledRole] = "enabled";
+    roles[InternalRole] = "internal";
     roles[PrimaryRole] = "primary";
     roles[SizeRole] = "size";
     roles[PositionRole] = "position";
     roles[NormalizedPositionRole] = "normalizedPosition";
+    roles[AutoRotateRole] = "autoRotate";
+    roles[AutoRotateOnlyInTabletModeRole] = "autoRotateOnlyInTabletMode";
     roles[RotationRole] = "rotation";
     roles[ScaleRole] = "scale";
     roles[ResolutionIndexRole] = "resolutionIndex";
@@ -365,6 +385,34 @@ bool OutputModel::setRefreshRate(int outputIndex, int refIndex)
     return true;
 }
 
+bool OutputModel::setAutoRotate(int outputIndex, bool value)
+{
+    Output &output = m_outputs[outputIndex];
+
+    if (m_config->autoRotate(output.ptr) == value) {
+        return false;
+    }
+    m_config->setAutoRotate(output.ptr, value);
+
+    QModelIndex index = createIndex(outputIndex, 0);
+    Q_EMIT dataChanged(index, index, {AutoRotateRole});
+    return true;
+}
+
+bool OutputModel::setAutoRotateOnlyInTabletMode(int outputIndex, bool value)
+{
+    Output &output = m_outputs[outputIndex];
+
+    if (m_config->autoRotateOnlyInTabletMode(output.ptr) == value) {
+        return false;
+    }
+    m_config->setAutoRotateOnlyInTabletMode(output.ptr, value);
+
+    QModelIndex index = createIndex(outputIndex, 0);
+    Q_EMIT dataChanged(index, index, {AutoRotateOnlyInTabletModeRole});
+    return true;
+}
+
 bool OutputModel::setRotation(int outputIndex, KScreen::Output::Rotation rotation)
 {
     const Output &output = m_outputs[outputIndex];
@@ -491,17 +539,27 @@ QVector<float> OutputModel::refreshRates(const KScreen::OutputPtr
     return hits;
 }
 
+int OutputModel::replicationSourceId(const Output &output) const
+{
+    const KScreen::OutputPtr source = m_config->replicationSource(output.ptr);
+    if (!source) {
+        return 0;
+    }
+    return source->id();
+}
+
 QStringList OutputModel::replicationSourceModel(const KScreen::OutputPtr &output) const
 {
     QStringList ret = { i18n("None") };
 
     for (const auto &out : m_outputs) {
         if (out.ptr->id() != output->id()) {
-            if (out.ptr->replicationSource() == output->id()) {
+            const int outSourceId = replicationSourceId(out);
+            if (outSourceId == output->id()) {
                 // 'output' is already source for replication, can't be replica itself
                 return { i18n("Replicated by other output") };
             }
-            if (out.ptr->replicationSource()) {
+            if (outSourceId) {
                 // This 'out' is a replica. Can't be a replication source.
                 continue;
             }
@@ -521,24 +579,26 @@ bool OutputModel::setReplicationSourceIndex(int outputIndex, int sourceIndex)
     }
 
     Output &output = m_outputs[outputIndex];
-    const int oldSourceId = output.ptr->replicationSource();
+    const int oldSourceId = replicationSourceId(output);
 
     if (sourceIndex < 0) {
         if (oldSourceId == 0) {
             // no change
             return false;
         }
-        output.ptr->setReplicationSource(0);
+        m_config->setReplicationSource(output.ptr, nullptr);
+        output.ptr->setLogicalSize(QSizeF());
         resetPosition(output);
     } else {
-        const int sourceId = m_outputs[sourceIndex].ptr->id();
-        if (oldSourceId == sourceId) {
+        const auto source = m_outputs[sourceIndex].ptr;
+        if (oldSourceId == source->id()) {
             // no change
             return false;
         }
-        output.ptr->setReplicationSource(sourceId);
+        m_config->setReplicationSource(output.ptr, source);
         output.posReset = output.ptr->pos();
-        output.ptr->setPos(m_outputs[sourceIndex].ptr->pos());
+        output.ptr->setPos(source->pos());
+        output.ptr->setLogicalSize(source->logicalSize());
     }
 
     reposition();
@@ -563,8 +623,12 @@ bool OutputModel::setReplicationSourceIndex(int outputIndex, int sourceIndex)
     return true;
 }
 
-int OutputModel::replicationSourceIndex(int outputIndex, int sourceId) const
+int OutputModel::replicationSourceIndex(int outputIndex) const
 {
+    const int sourceId = replicationSourceId(m_outputs[outputIndex]);
+    if (!sourceId) {
+        return 0;
+    }
     for (int i = 0; i < m_outputs.size(); i++) {
         const Output &output = m_outputs[i];
         if (output.ptr->id() == sourceId) {
@@ -580,7 +644,7 @@ QVariantList OutputModel::replicasModel(const KScreen::OutputPtr &output) const
     for (int i = 0; i < m_outputs.size(); i++) {
         const Output &out = m_outputs[i];
         if (out.ptr->id() != output->id()) {
-            if (out.ptr->replicationSource() == output->id()) {
+            if (replicationSourceId(out) == output->id()) {
                 ret << i;
             }
         }
