@@ -17,7 +17,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "config.h"
 #include "output.h"
-#include "../common/globals.h"
 #include "../common/control.h"
 #include "kscreen_daemon_debug.h"
 #include "device.h"
@@ -39,8 +38,10 @@ QString Config::configsDirPath()
     return Globals::dirPath() % s_configsDirName;
 }
 
-Config::Config(KScreen::ConfigPtr config)
-    : m_data(config)
+Config::Config(KScreen::ConfigPtr config, QObject *parent)
+    : QObject(parent)
+    , m_data(config)
+    , m_control(new ControlConfig(config, this))
 {
 }
 
@@ -60,6 +61,39 @@ QString Config::id() const
     return m_data->connectedOutputsHash();
 }
 
+void Config::activateControlWatching()
+{
+    connect(m_control, &ControlConfig::changed, this, &Config::controlChanged);
+    m_control->activateWatcher();
+}
+
+bool Config::autoRotationRequested() const
+{
+    for (KScreen::OutputPtr &output : m_data->outputs()) {
+        if (m_control->getAutoRotate(output)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+void Config::setDeviceOrientation(QOrientationReading::Orientation orientation)
+{
+    for (KScreen::OutputPtr &output : m_data->outputs()) {
+        if (!m_control->getAutoRotate(output)) {
+            continue;
+        }
+        auto finalOrientation = orientation;
+        if (m_control->getAutoRotateOnlyInTabletMode(output) && !m_data->tabletModeEngaged()) {
+            finalOrientation = QOrientationReading::Orientation::TopUp;
+        }
+        if (Output::updateOrientation(output, finalOrientation)) {
+            // TODO: call Layouter to find fitting positions for other outputs again
+            return;
+        }
+    }
+}
+
 bool Config::fileExists() const
 {
     return (QFile::exists(configsDirPath() % id()) || QFile::exists(configsDirPath() % s_fixedConfigFileName));
@@ -169,8 +203,6 @@ bool Config::writeFile(const QString &filePath)
     }
     const KScreen::OutputList outputs = m_data->outputs();
 
-    const auto control = ControlConfig(m_data);
-
     const auto oldConfig = readFile();
     KScreen::OutputList oldOutputs;
     if (oldConfig) {
@@ -201,11 +233,6 @@ bool Config::writeFile(const QString &filePath)
             if (!out) {
                 return;
             }
-            QString replicationSourceHash;
-            if (int sourceId = out->replicationSource()) {
-                replicationSourceHash = m_data->output(sourceId)->hashMd5();
-            }
-            info[QStringLiteral("replicate")] = replicationSourceHash;
 
             QVariantMap pos;
             pos[QStringLiteral("x")] = out->pos().x();
@@ -215,7 +242,7 @@ bool Config::writeFile(const QString &filePath)
         setOutputConfigInfo(output->isEnabled() ? output : oldOutput);
 
         if (output->isEnabled() &&
-                control.getOutputRetention(output->hash(), output->name()) !=
+                m_control->getOutputRetention(output->hash(), output->name()) !=
                     Control::OutputRetention::Individual) {
             // try to update global output data
             Output::writeGlobal(output);
