@@ -16,7 +16,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "output.h"
 #include "config.h"
-#include "../common/globals.h"
 
 #include "kscreen_daemon_debug.h"
 #include "generator.h"
@@ -119,6 +118,45 @@ bool Output::readInGlobal(KScreen::OutputPtr output)
     return true;
 }
 
+KScreen::Output::Rotation orientationToRotation(QOrientationReading::Orientation orientation,
+                                                KScreen::Output::Rotation fallback)
+{
+    using Orientation = QOrientationReading::Orientation;
+
+    switch (orientation) {
+    case Orientation::TopUp:
+        return KScreen::Output::Rotation::None;
+    case Orientation::TopDown:
+        return KScreen::Output::Rotation::Inverted;
+    case Orientation::LeftUp:
+        return KScreen::Output::Rotation::Right;
+    case Orientation::RightUp:
+        return KScreen::Output::Rotation::Left;
+    case Orientation::Undefined:
+    case Orientation::FaceUp:
+    case Orientation::FaceDown:
+        return fallback;
+    default:
+        Q_UNREACHABLE();
+    }
+}
+
+bool Output::updateOrientation(KScreen::OutputPtr &output,
+                               QOrientationReading::Orientation orientation)
+{
+    if (output->type() != KScreen::Output::Type::Panel) {
+        return false;
+    }
+    const auto currentRotation = output->rotation();
+    const auto rotation = orientationToRotation(orientation, currentRotation);
+    if (rotation == currentRotation) {
+        return true;
+    }
+    output->setRotation(rotation);
+    return true;
+}
+
+// TODO: move this into the Layouter class.
 void Output::adjustPositions(KScreen::ConfigPtr config, const QVariantList &outputsInfo)
 {
     typedef QPair<int, QPoint> Out;
@@ -212,7 +250,7 @@ void Output::adjustPositions(KScreen::ConfigPtr config, const QVariantList &outp
         const int xCorrected = prevRight + prevGeo.width() * xInfoDiff / (double)prevInfoGeo.width();
         const int xDiff = curGeo.x() - prevRight;
 
-        // In the following calculate the y-coorection. This is more involved since we
+        // In the following calculate the y-correction. This is more involved since we
         // differentiate between overlapping and non-overlapping pairs and align either
         // top to top/bottom or bottom to top/bottom
         const bool yOverlap = prevInfoGeo.y() + prevInfoGeo.height() > curInfoGeo.y() &&
@@ -327,22 +365,10 @@ void Output::readInOutputs(KScreen::ConfigPtr config, const QVariantList &output
             }
             infoFound = true;
             readIn(output, info, control.getOutputRetention(output));
-
-            const QString replicationSourceHash = info[QStringLiteral("replicate")].toString();
-            if (replicationSourceHash.isEmpty()) {
-                output->setReplicationSource(0);
-            } else {
-                for (const KScreen::OutputPtr out : outputs) {
-                    if (out != output && out->hashMd5() == replicationSourceHash) {
-                        output->setReplicationSource(out->id());
-                        break;
-                    }
-                }
-            }
             break;
         }
         if (!infoFound) {
-            // no info in info for this output, try reading in global output info atleast or set some default values
+            // no info in info for this output, try reading in global output info at least or set some default values
 
             qCWarning(KSCREEN_KDED) << "\tFailed to find a matching output in the current info data - this means that our info is corrupted"
                                        "or a different device with the same serial number has been connected (very unlikely).";
@@ -352,8 +378,22 @@ void Output::readInOutputs(KScreen::ConfigPtr config, const QVariantList &output
             }
         }
     }
+
+    for (KScreen::OutputPtr output : outputs) {
+        auto replicationSource = control.getReplicationSource(output);
+        if (replicationSource) {
+            output->setPos(replicationSource->pos());
+            output->setLogicalSize(replicationSource->logicalSize());
+        } else {
+            output->setLogicalSize(QSizeF());
+        }
+    }
+
+    // TODO: this does not work at the moment with logical size replication. Deactivate for now.
     // correct positional config regressions on global output data changes
+#if 0
     adjustPositions(config, outputsInfo);
+#endif
 }
 
 static QVariantMap metadata(const KScreen::OutputPtr &output)
@@ -376,8 +416,8 @@ bool Output::writeGlobalPart(const KScreen::OutputPtr &output, QVariantMap &info
     info[QStringLiteral("metadata")] = metadata(output);
     info[QStringLiteral("rotation")] = output->rotation();
 
-    // Round scale to one digit.
-    info[QStringLiteral("scale")] = int(output->scale() * 10 + 0.5) / 10.;
+    // Round scale to four digits
+    info[QStringLiteral("scale")] = int(output->scale() * 10000 + 0.5) / 10000.;
 
     QVariantMap modeInfo;
     float refreshRate = -1.;
